{
  (C) 2003 Dirk Frischalowski, dfrischalowski@del-net.com, all rights reserved.

  Version 1.0 25.02.2003

  Version 2.0 09.02.2004
  - new, more Infos for FileDescription, FileVersion etc.
}

unit DFFileInfo;

interface

uses
  Windows, SysUtils, Classes;

type
  TDFFileInfo = class(TComponent)
  private
    // for Values see Windows help for VS_FIXEDFILEINFO
    FMajorFileVersionMS: word; // A part of A.B.C.D
    FMinorFileVersionMS: word; // B part of A.B.C.D - 2 digits with leading zero
    FMajorFileVersionLS: word; // C part of A.B.C.D
    FMinorFileVersionLS: word; // D part of A.B.C.D
    FMajorProductVersionMS: word;
    FMinorProductVersionMS: word;
    FMajorProductVersionLS: word;
    FMinorProductVersionLS: word;
    FFileType: dword;
    FFileTypeAsString: string;
    FFileSubType: dword;
    FFileSubTypeAsString: string;
    FFileOS: dword;
    FFileOSAsString: string;
    FMajorFileDataMS: word;
    FMinorFileDataMS: word;
    FMajorFileDataLS: word;
    FMinorFileDataLS: word;
    FFileFlagsMask: dword;
    FFileFlags: dword;
    FIsFlagDebug: boolean;
    FIsFlagInfoinferred: boolean;
    FIsFlagPatched: boolean;
    FIsFlagPreRelease: boolean;
    FIsFlagPrivateBuild: boolean;
    FIsFlagSpecialBuild: boolean;
    FIsVersionInfo: boolean;
    FFileName: string;
    FCompanyName: string;
    FFileDesciption: string;
    FFileVersion: string;
    FInternalName: string;
    FLegalCopyright: string;
    FLegalTrademarks: string;
    FOriginalFilename: string;
    FProductName: string;
    FProductVersion: string;
    FComments: string;
    FSpecialBuild: string;
    FPrivateBuild: string;
    procedure Init;
    procedure SetFileName(FileName: string);
    procedure SetFileSubTypeAsString;
    procedure SetFileOSAsString;
  protected
  public
    constructor Create(AOwner: TComponent); override;
    function GetSystemPath: string; // helper functions for finding dlls ...
    function GetWindowsPath: string; // helper functions for finding dlls ...
  published
    property FileName: string read FFileName write SetFileName;
    property IsVersionInfo: boolean read FIsVersionInfo default false;
    property MajorFileVersionMS: word read FMajorFileVersionMS default 0;
    property MinorFileVersionMS: word read FMinorFileVersionMS default 0;
    property MajorFileVersionLS: word read FMajorFileVersionLS default 0;
    property MinorFileVersionLS: word read FMinorFileVersionLS default 0;
    property MajorProductVersionMS: word read FMajorProductVersionMS default 0;
    property MinorProductVersionMS: word read FMinorProductVersionMS default 0;
    property MajorProductVersionLS: word read FMajorProductVersionLS default 0;
    property MinorProductVersionLS: word read FMinorProductVersionLS default 0;
    property FileType: dword read FFileType default VFT_UNKNOWN;
    property FileSubType: dword read FFilesubType default VFT2_UNKNOWN;
    property MajorFileDataMS: word read FMajorFileDataMS default 0;
    property MinorFileDataMS: word read FMinorFileDataMS default 0;
    property MajorFileDataLS: word read FMajorFileDataLS default 0;
    property MinorFileDataLS: word read FMinorFileDataLS default 0;
    property FileOS: dword read FFileOS default VOS_UNKNOWN;
    property IsFlagDebug: boolean read FIsFlagDebug default false;
    property IsFlagInfoinferred: boolean read FIsFlagInfoinferred default false;
    property IsFlagPatched: boolean read FIsFlagPatched default false;
    property IsFlagPreRelease: boolean read FIsFlagPreRelease default false;
    property IsFlagPrivateBuild: boolean read FIsFlagPrivateBuild default false;
    property IsFlagSpecialBuild: boolean read FIsFlagSpecialBuild default false;
    property FileFlagsMask: dword read FFileFlagsMask default 0;
    property FileFlags: dword read FFileFlags default 0;
    property FileTypeAsString: string read FFileTypeAsString;
    property FileSubTypeAsString: string read FFileSubTypeAsString;
    property FileOSAsString: string read FFileOSAsString;

    property CompanyName: string read FCompanyName;
    property FileDesciption: string read FFileDesciption;
    property FileVersion: string read FFileVersion;
    property InternalName: string read FInternalName;
    property LegalCopyright: string read FLegalCopyright;
    property LegalTrademarks: string read FLegalTrademarks;
    property OriginalFilename: string read FOriginalFilename;
    property ProductName: string read FProductName;
    property ProductVersion: string read FProductVersion;
    property Comments: string read FComments;
    property SpecialBuild: string read FSpecialBuild;
    property PrivateBuild: string read FPrivateBuild;
  end;

procedure Register;

implementation

procedure Register;
begin
  RegisterComponents('DFrisch', [TDFFileInfo]);
end;

{ TDFFileInfo }

constructor TDFFileInfo.Create(AOwner: TComponent);
begin
  inherited;
  Init;
end;

function TDFFileInfo.GetSystemPath: string;
var
  TempDirectory: array[0..255] of char;
begin
  Result := '';
  GetSystemDirectory(TempDirectory, 255);
  Result := StrPas(TempDirectory);
  if Result[Length(Result)] = '\' then
    Delete(Result, Length(Result), 1);
end;

function TDFFileInfo.GetWindowsPath: string;
var
  TempDirectory: array[0..255] of char;
begin
  Result := '';
  GetWindowsDirectory(TempDirectory, 255);
  Result := StrPas(TempDirectory);
  if Result[Length(Result)] = '\' then
    Delete(Result, Length(Result), 1);
end;

procedure TDFFileInfo.Init;
begin
  FIsVersionInfo := false;
  FMajorFileVersionMS := 0;
  FMinorFileVersionMS := 0;
  FMajorFileVersionLS := 0;
  FMinorFileVersionLS := 0;
  FMajorProductVersionMS := 0;
  FMinorProductVersionMS := 0;
  FMajorProductVersionLS := 0;
  FMinorProductVersionLS := 0;
  FFileType := VFT_UNKNOWN;
  FFileSubType := VFT2_UNKNOWN;
  FMajorFileDataMS := 0;
  FMinorFileDataMS := 0;
  FMajorFileDataLS := 0;
  FMinorFileDataLS := 0;
  FFileOS := VOS_UNKNOWN;
  FIsFlagDebug := false;
  FIsFlagInfoinferred := false;
  FIsFlagPatched := false;
  FIsFlagPreRelease := false;
  FIsFlagPrivateBuild := false;
  FIsFlagSpecialBuild := false;
  FFileFlagsMask := 0;
  FFileFlags := 0;
  FFileName := '';
  FCompanyName := '';
  FFileDesciption := '';
  FFileVersion := '';
  FInternalName := '';
  FLegalCopyright := '';
  FLegalTrademarks := '';
  FOriginalFilename := '';
  FProductName := '';
  FProductVersion := '';
  FComments := '';
  FSpecialBuild := '';
  FPrivateBuild := '';
end;

procedure TDFFileInfo.SetFileName(FileName: string);
var
  FileInfo: PVSFixedFileInfo;
  FDummy: dword;
  BufferSize: dword;
  Buffer: Pointer;
  {$ifdef VER120}
  FileInfoSize: Cardinal;
  {$else}
  FileInfoSize: uint;
  {$endif}

  LangCharSetInfo: PInteger;
  LangCharSet: string;

  Len: Cardinal;

  function GetValue(InfoStr: string): string;
  var
    TmpStr: Pointer;
    TmpStrSize: Cardinal;
  begin
    Result := '';
    if VerQueryValue(Buffer,
                     PChar('\StringFileInfo\' + LangCharSet + '\' + InfoStr),
                     TmpStr, Len) then
      Result := StrPas(TmpStr);
  end;
begin
  Init;

  if FFileName <> FileName then
  begin
    if FileExists(FileName) then
    begin
      BufferSize := GetFileVersionInfoSize(PChar(FileName), FDummy);
      if BufferSize <> 0 then
      begin
        GetMem(Buffer, BufferSize);
        try
          if GetFileVersionInfo(PChar(FileName), FDummy, BufferSize, Buffer) then
          begin

            if VerQueryValue(Buffer, '\', Pointer(FileInfo), FileInfoSize) then
            begin
              if FileInfo^.dwSignature = $feef04bd then
              begin
                FMajorFileVersionMS := HiWord(FileInfo^.dwFileVersionMS);
                FMinorFileVersionMS := LoWord(FileInfo^.dwFileVersionMS);
                FMajorFileVersionLS := HiWord(FileInfo^.dwFileVersionLS);
                FMinorFileVersionLS := LoWord(FileInfo^.dwFileVersionLS);
                FMajorProductVersionMS := HiWord(FileInfo^.dwProductVersionMS);
                FMinorProductVersionMS := LoWord(FileInfo^.dwProductVersionMS);
                FMajorProductVersionLS := HiWord(FileInfo^.dwProductVersionLS);
                FMinorProductVersionLS := LoWord(FileInfo^.dwProductVersionLS);
                FMajorFileDataMS := HiWord(FileInfo^.dwFileDateMS);
                FMinorFileDataMS := LoWord(FileInfo^.dwFileDateMS);
                FMajorFileDataLS := HiWord(FileInfo^.dwFileDateLS);
                FMinorFileDataLS := LoWord(FileInfo^.dwFileDateLS);
                FFileOS := FileInfo^.dwFileOS;
                FFileType := FileInfo^.dwFileType;
                FFileSubType := FileInfo^.dwFileSubtype;
                FFileFlagsMask := FileInfo^.dwFileFlagsMask;
                FFileFlags := FileInfo^.dwFileFlags;
                FIsFlagDebug := (FFileFlagsMask and FFileFlags and VS_FF_DEBUG) > 0;
                FIsFlagInfoinferred := (FFileFlagsMask and FFileFlags and VS_FF_INFOINFERRED) > 0;
                FIsFlagPatched := (FFileFlagsMask and FFileFlags and VS_FF_PATCHED) > 0;
                FIsFlagPreRelease := (FFileFlagsMask and FFileFlags and VS_FF_PRERELEASE) > 0;
                FIsFlagPrivateBuild := (FFileFlagsMask and FFileFlags and VS_FF_PRIVATEBUILD) > 0;
                FIsFlagSpecialBuild := (FFileFlagsMask and FFileFlags and VS_FF_SPECIALBUILD) > 0;
                SetFileSubTypeAsString;
                SetFileOSAsString;
                FIsVersionInfo := true;
              end;
            end;

            if VerQueryValue(Buffer, '\VarFileInfo\Translation', Pointer(LangCharSetInfo), Len) then
            begin
              LangCharSet := Format('%.4x%.4x', [LoWord(LangCharSetInfo^), HiWord(LangCharSetInfo^)]);

              FFileDesciption := GetValue('FileDescription');
              FCompanyName := GetValue('CompanyName');
              FFileVersion := GetValue('FileVersion');
              FInternalName := GetValue('InternalName');
              FLegalCopyright := GetValue('LegalCopyright');
              FLegalTrademarks := GetValue('LegalTrademarks');
              FOriginalFilename := GetValue('OriginalFilename');
              FProductName := GetValue('ProductName');
              FProductVersion := GetValue('ProductVersion');
              FComments := GetValue('Comments');
              FSpecialBuild := GetValue('SpecialBuild');
              FPrivateBuild := GetValue('PrivateBuild');
            end;

          end;
        finally
          FreeMem(Buffer, BufferSize);
        end;
      end;
    end;
  end;
end;

procedure TDFFileInfo.SetFileOSAsString;
begin
  case FileOS of
    VOS_UNKNOWN:       FFileOSAsString := 'VOS_UNKNOWN';
    VOS_DOS:           FFileOSAsString := 'VOS_DOS';
    VOS_OS216:         FFileOSAsString := 'VOS_OS216';
    VOS_OS232:         FFileOSAsString := 'VOS_OS232';
    VOS_NT:            FFileOSAsString := 'VOS_NT';
    VOS__WINDOWS16:    FFileOSAsString := 'VOS__WINDOWS16';
    VOS__PM16:         FFileOSAsString := 'VOS__PM16';
    VOS__PM32:         FFileOSAsString := 'VOS__PM32';
    VOS__WINDOWS32:    FFileOSAsString := 'VOS__WINDOWS32';
    VOS_DOS_WINDOWS16: FFileOSAsString := 'VOS_DOS_WINDOWS16';
    VOS_DOS_WINDOWS32: FFileOSAsString := 'VOS_DOS_WINDOWS32';
    VOS_OS216_PM16:    FFileOSAsString := 'VOS_OS216_PM16';
    VOS_OS232_PM32:    FFileOSAsString := 'VOS_OS232_PM32';
    VOS_NT_WINDOWS32:  FFileOSAsString := 'VOS_NT_WINDOWS32';
  end;
end;

procedure TDFFileInfo.SetFileSubTypeAsString;
begin
  case FileType of
    VFT_UNKNOWN: FFileTypeAsString := 'VFT_UNKNOWN';
    VFT_APP: FFileTypeAsString := 'VFT_APP';
    VFT_DLL: FFileTypeAsString := 'VFT_DLL';
    VFT_DRV: begin
               FFileTypeAsString := 'VFT_DRV';
               case FileSubType of
                   VFT2_UNKNOWN:         FFileSubTypeAsString := 'VFT2_UNKNOWN';
                   VFT2_DRV_PRINTER:     FFileSubTypeAsString := 'VFT2_DRV_PRINTER';
                   VFT2_DRV_KEYBOARD:    FFileSubTypeAsString := 'VFT2_DRV_KEYBOARD';
                   VFT2_DRV_LANGUAGE:    FFileSubTypeAsString := 'VFT2_DRV_LANGUAGE';
                   VFT2_DRV_DISPLAY:     FFileSubTypeAsString := 'VFT2_DRV_DISPLAY';
                   VFT2_DRV_MOUSE:       FFileSubTypeAsString := 'VFT2_DRV_MOUSE';
                   VFT2_DRV_NETWORK:     FFileSubTypeAsString := 'VFT2_DRV_NETWORK';
                   VFT2_DRV_SYSTEM:      FFileSubTypeAsString := 'VFT2_DRV_SYSTEM';
                   VFT2_DRV_INSTALLABLE: FFileSubTypeAsString := 'VFT2_DRV_INSTALLABLE';
                   VFT2_DRV_SOUND:       FFileSubTypeAsString := 'VFT2_DRV_SOUND';
                   VFT2_DRV_COMM:        FFileSubTypeAsString := 'VFT2_DRV_COMM';
               end;
             end;
    VFT_FONT: begin
                FFileTypeAsString := 'VFT_FONT';
                case FileSubType of
                  VFT2_FONT_RASTER:   FFileSubTypeAsString := 'VFT2_FONT_RASTER';
                  VFT2_FONT_VECTOR:   FFileSubTypeAsString := 'VFT2_FONT_VECTOR';
                  VFT2_FONT_TRUETYPE: FFileSubTypeAsString := 'VFT2_FONT_TRUETYPE';
                end;
              end;
    VFT_VXD:        FFileTypeAsString := 'VFT_VXD';
    VFT_STATIC_LIB: FFileTypeAsString := 'VFT_STATIC_LIB';
  end;
end;

end.


